@@ -1,4 +1,4 @@
-/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2002 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2002, 2006 Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -12,7 +12,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <assert.h> 
@@ -89,7 +89,7 @@ scm_i_initialize_heap_segment_data (scm_t_heap_segment * segment, size_t request
   scm_t_cell *  memory = 0;
 
   /*
-    We use malloc to alloc the heap. On GNU libc this is 
+    We use calloc to alloc the heap. On GNU libc this is 
     equivalent to mmapping /dev/zero
    */
   SCM_SYSCALL (memory = (scm_t_cell * ) calloc (1, mem_needed));
@@ -170,8 +170,13 @@ scm_i_sweep_some_cards (scm_t_heap_segment *seg)
   scm_gc_cells_collected += collected * seg->span;
 
   if (!seg->first_time)
-    scm_cells_allocated -= collected * seg->span;
-  
+    {
+      scm_gc_cells_allocated_acc +=
+	(scm_cells_allocated - scm_last_cells_allocated);
+
+      scm_cells_allocated -= collected * seg->span;
+      scm_last_cells_allocated = scm_cells_allocated;
+    }
   seg->freelist->collected += collected  * seg->span;
   
 
@@ -202,14 +207,24 @@ scm_i_sweep_segment (scm_t_heap_segment * seg)
   int yield = scm_gc_cells_collected;
   int coll = seg->freelist->collected;
   unsigned long alloc = scm_cells_allocated ;
+  unsigned long last_alloc = scm_last_cells_allocated;
+  double last_total
+    = scm_gc_cells_allocated_acc
+    + (alloc - last_alloc);
   
   while (scm_i_sweep_some_cards (seg) != SCM_EOL)
     ;
-
-  scm_gc_cells_collected = yield;
-  scm_cells_allocated = alloc;
-  seg->freelist->collected = coll; 
   
+  scm_gc_cells_collected = yield;
+
+  /*
+   * restore old stats. 
+   */
+  scm_gc_cells_allocated_acc = last_total;
+  scm_cells_allocated = alloc;
+  scm_last_cells_allocated = alloc;
+
+  seg->freelist->collected = coll; 
   seg->next_free_card =p;
 }
 
@@ -320,7 +335,7 @@ SCM
 scm_i_sweep_some_segments (scm_t_cell_type_statistics * fl)
 {
   int i = fl->heap_segment_idx;
-  SCM collected =SCM_EOL;
+  SCM collected = SCM_EOL;
   
   if (i == -1)
     i++;
@@ -458,19 +473,11 @@ scm_i_find_heap_segment_containing_object (SCM obj)
   RETURN: the index of the segment.
  */
 int 
-scm_i_get_new_heap_segment (scm_t_cell_type_statistics *freelist, policy_on_error error_policy)
+scm_i_get_new_heap_segment (scm_t_cell_type_statistics *freelist,
+			    policy_on_error error_policy)
 {
   size_t len;
 
-  if (scm_gc_heap_lock)
-    {
-      /* Critical code sections (such as the garbage collector) aren't
-       * supposed to add heap segments.
-       */
-      fprintf (stderr, "scm_i_get_new_heap_segment: Can not extend locked heap.\n");
-      abort ();
-    }
-
   {
     /* Assure that the new segment is predicted to be large enough.
      *
