@@ -1,4 +1,4 @@
-/*      Copyright (C) 1995,1996,1997,1998, 2000, 2001 Free Software Foundation, Inc.
+/*      Copyright (C) 1995,1996,1997,1998, 2000, 2001, 2006 Free Software Foundation, Inc.
 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -12,7 +12,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 
@@ -21,6 +21,8 @@
 
 #include "libguile/gh.h"
 
+#if SCM_ENABLE_DEPRECATED
+
 /* allows you to define new scheme primitives written in C */
 SCM
 gh_new_procedure (const char *proc_name, SCM (*fn) (),
@@ -143,6 +145,8 @@ gh_call3 (SCM proc, SCM arg1, SCM arg2, SCM arg3)
   return scm_apply (proc, arg1, scm_cons2 (arg2, arg3, scm_listofnull));
 }
 
+#endif /* SCM_ENABLE_DEPRECATED */
+
 /*
   Local Variables:
   c-file-style: "gnu"
