@@ -487,11 +487,25 @@ SCM_DEFINE (scm_kill, "kill", 2, 0, 0,
   /* Signal values are interned in scm_init_posix().  */
 #ifdef HAVE_KILL
   if (kill (scm_to_int (pid), scm_to_int  (sig)) != 0)
+    SCM_SYSERROR;
 #else
+  /* Mingw has raise(), but not kill().  (Other raw DOS environments might
+     be similar.)  Use raise() when the requested pid is our own process,
+     otherwise bomb.  */
   if (scm_to_int (pid) == getpid ())
-    if (raise (scm_to_int (sig)) != 0)
+    {
+      if (raise (scm_to_int (sig)) != 0)
+        {
+        err:
+          SCM_SYSERROR;
+        }
+      else
+        {
+          errno = ENOSYS;
+          goto err;
+        }
+    }
 #endif
-      SCM_SYSERROR;
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -820,7 +834,7 @@ SCM_DEFINE (scm_ttyname, "ttyname", 1, 0, 0,
 {
   char *result;
   int fd, err;
-  SCM ret;
+  SCM ret = SCM_BOOL_F;
 
   port = SCM_COERCE_OUTPORT (port);
   SCM_VALIDATE_OPPORT (1, port);
@@ -829,9 +843,12 @@ SCM_DEFINE (scm_ttyname, "ttyname", 1, 0, 0,
   fd = SCM_FPORT_FDES (port);
 
   scm_i_scm_pthread_mutex_lock (&scm_i_misc_mutex);
+
   SCM_SYSCALL (result = ttyname (fd));
   err = errno;
-  ret = scm_from_locale_string (result);
+  if (result != NULL)
+    result = strdup (result);
+
   scm_i_pthread_mutex_unlock (&scm_i_misc_mutex);
 
   if (!result)
@@ -839,6 +856,9 @@ SCM_DEFINE (scm_ttyname, "ttyname", 1, 0, 0,
       errno = err;
       SCM_SYSERROR;
     }
+  else
+    ret = scm_take_locale_string (result);
+
   return ret;
 }
 #undef FUNC_NAME
@@ -1312,9 +1332,6 @@ SCM_DEFINE (scm_putenv, "putenv", 1, 0, 0,
 {
   int rv;
   char *c_str = scm_to_locale_string (str);
-#ifdef __MINGW32__
-  size_t len = strlen (c_str);
-#endif
 
   if (strchr (c_str, '=') == NULL)
     {
@@ -1329,6 +1346,7 @@ SCM_DEFINE (scm_putenv, "putenv", 1, 0, 0,
       /* On e.g. Win32 hosts putenv() called with 'name=' removes the
 	 environment variable 'name'. */
       int e;
+      size_t len = strlen (c_str);
       char *ptr = scm_malloc (len + 2);
       strcpy (ptr, c_str);
       strcpy (ptr+len, "=");
@@ -1348,26 +1366,29 @@ SCM_DEFINE (scm_putenv, "putenv", 1, 0, 0,
          by getenv.  It's not enough just to modify the string we set,
          because MINGW putenv copies it.  */
 
-      if (c_str[len-1] == '=')
-        {
-	  char *ptr = scm_malloc (len+2);
-	  strcpy (ptr, c_str);
-	  strcpy (ptr+len, " ");
-	  rv = putenv (ptr);
-	  if (rv < 0)
-	    {
-	      int eno = errno;
-	      free (c_str);
-	      errno = eno;
-	      SCM_SYSERROR;
-	    }
-	  /* truncate to just the name */
-	  c_str[len-1] = '\0';
-	  ptr = getenv (c_str);
-	  if (ptr)
-	    ptr[0] = '\0';
-	  return SCM_UNSPECIFIED;
-        }
+      {
+        size_t len = strlen (c_str);
+        if (c_str[len-1] == '=')
+          {
+            char *ptr = scm_malloc (len+2);
+            strcpy (ptr, c_str);
+            strcpy (ptr+len, " ");
+            rv = putenv (ptr);
+            if (rv < 0)
+              {
+                int eno = errno;
+                free (c_str);
+                errno = eno;
+                SCM_SYSERROR;
+              }
+            /* truncate to just the name */
+            c_str[len-1] = '\0';
+            ptr = getenv (c_str);
+            if (ptr)
+              ptr[0] = '\0';
+            return SCM_UNSPECIFIED;
+          }
+      }
 #endif /* __MINGW32__ */
 
       /* Leave c_str in the environment.  */
@@ -1546,7 +1567,7 @@ SCM_DEFINE (scm_crypt, "crypt", 2, 0, 0,
 #define FUNC_NAME s_scm_crypt
 {
   SCM ret;
-  char *c_key, *c_salt;
+  char *c_key, *c_salt, *c_ret;
 
   scm_dynwind_begin (0);
   scm_i_dynwind_pthread_mutex_lock (&scm_i_misc_mutex);
@@ -1556,8 +1577,14 @@ SCM_DEFINE (scm_crypt, "crypt", 2, 0, 0,
   c_salt = scm_to_locale_string (salt);
   scm_dynwind_free (c_salt);
 
-  ret = scm_from_locale_string (crypt (c_key, c_salt));
+  /* The Linux crypt(3) man page says crypt will return NULL and set errno
+     on error.  (Eg. ENOSYS if legal restrictions mean it cannot be
+     implemented).  */
+  c_ret = crypt (c_key, c_salt);
+  if (c_ret == NULL)
+    SCM_SYSERROR;
 
+  ret = scm_from_locale_string (c_ret);
   scm_dynwind_end ();
   return ret;
 }
