@@ -1,5 +1,5 @@
 ;;;; test-suite/lib.scm --- generic support for testing
-;;;; Copyright (C) 1999, 2000, 2001, 2004, 2006 Free Software Foundation, Inc.
+;;;; Copyright (C) 1999, 2000, 2001, 2004, 2006, 2007 Free Software Foundation, Inc.
 ;;;;
 ;;;; This program is free software; you can redistribute it and/or modify
 ;;;; it under the terms of the GNU General Public License as published by
@@ -29,7 +29,9 @@
  exception:wrong-num-args exception:wrong-type-arg
  exception:numerical-overflow
  exception:struct-set!-denied
+ exception:system-error
  exception:miscellaneous-error
+ exception:string-contains-nul
 
  ;; Reporting passes and failures.
  run-test
@@ -256,9 +258,16 @@
   (cons 'numerical-overflow "^Numerical overflow"))
 (define exception:struct-set!-denied
   (cons 'misc-error "^set! denied for field"))
+(define exception:system-error
+  (cons 'system-error ".*"))
 (define exception:miscellaneous-error
   (cons 'misc-error "^.*"))
 
+;; as per throw in scm_to_locale_stringn()
+(define exception:string-contains-nul
+  (cons 'misc-error "^string contains #\\\\nul character"))
+
+
 ;;; Display all parameters to the default output port, followed by a newline.
 (define (display-line . objs)
   (for-each display objs)
